@@ -7,7 +7,7 @@ use rr_util::{
 };
 use uuid::uuid;
 
-use crate::{CachedCircuitInfo, CircuitNode, ConstructError, NamedAxes, Result};
+use crate::{CachedCircuitInfo, CircuitFlags, CircuitNode, ConstructError, NamedAxes, Result};
 
 pub trait CircuitNodeInit {
     fn init_info_impl(self) -> Result<Self>
@@ -31,15 +31,10 @@ pub trait CircuitNodePrivate {
 
 pub trait CircuitNodeComputeInfoImpl: CircuitNode {
     fn compute_shape(&self) -> Shape;
-    fn compute_is_constant(&self) -> bool {
-        self.children().all(|c| c.info().is_constant)
-    }
-    fn compute_is_explicitly_computable(&self) -> bool {
-        self.children().all(|c| c.info().is_explicitly_computable)
-    }
-    fn compute_can_be_sampled(&self) -> bool {
-        self.children().all(|c| c.info().can_be_sampled)
+    fn compute_flags(&self) -> CircuitFlags {
+        self.compute_flags_default()
     }
+
     fn device_dtype_extra(&self) -> Box<dyn Iterator<Item = TorchDeviceDtypeOp> + '_> {
         Box::new(std::iter::empty())
     }
@@ -97,9 +92,7 @@ pub trait CircuitNodeSetNonHashInfo: CircuitNodePrivate {
 impl<T: CircuitNodeComputeInfoImpl + CircuitNodePrivate> CircuitNodeSetNonHashInfo for T {
     fn set_non_hash_info(&mut self) -> Result<()> {
         self.info_mut().shape = self.compute_shape(); // set shape so methods to compute other self.info_mut() can use it
-        self.info_mut().is_constant = self.compute_is_constant();
-        self.info_mut().is_explicitly_computable = self.compute_is_explicitly_computable();
-        self.info_mut().can_be_sampled = self.compute_can_be_sampled();
+        self.info_mut().flags = self.compute_flags();
         self.info_mut().device_dtype = self.compute_device_dtype()?;
         self.info_mut().symbolic_size_constraints = self.compute_symbolic_size_constraints()?;
         self.info_mut().named_axes = self.compute_named_axes();
