@@ -9,6 +9,7 @@ use rr_util::{
     atr,
     rearrange_spec::RInnerInts,
     tensor_util::{IndexError, IndexTensor, TensorIndexSync},
+    tu8v,
     util::EinsumAxes,
 };
 use rr_util::{
@@ -392,7 +393,7 @@ impl Einsum {
         Einsum::try_new(
             vec![
                 (node, axes.clone()),
-                (Scalar::new(scalar, sv![], scalar_name).rc(), sv![]),
+                (Scalar::new(scalar, sv![], scalar_name).rc(), tu8v![]),
             ],
             axes,
             name,
@@ -439,7 +440,7 @@ impl Einsum {
     }
     #[staticmethod]
     pub fn empty(name: Option<String>) -> Self {
-        Einsum::try_new(vec![], sv![], name).unwrap()
+        Einsum::try_new(vec![], tu8v![], name).unwrap()
     }
 
     #[staticmethod]
@@ -1198,9 +1199,9 @@ impl Index {
         let rearranged_tensors_idxs_first = Rearrange::try_new(
             node.clone(),
             RearrangeSpec {
-                input_ints: (0..index.0.len() as u8).map(|x| sv![x]).collect(),
+                input_ints: (0..index.0.len() as u8).map(|x| tu8v![x]).collect(),
                 output_ints: std::iter::once(tensor_part_is.clone())
-                    .chain(nontensor_part_is.iter().map(|x| sv![*x]))
+                    .chain(nontensor_part_is.iter().map(|x| tu8v![*x]))
                     .collect(),
                 int_sizes: sv![OpSize::NONE;index.0.len()],
             },
@@ -1267,11 +1268,11 @@ impl Index {
                     .chain(
                         (broadcasted_shape.len() as u8
                             ..broadcasted_shape.len() as u8 + nontensor_part_is.len() as u8)
-                            .map(|x| sv![x]),
+                            .map(|x| tu8v![x]),
                     )
                     .collect(),
                 output_ints: (0..broadcasted_shape.len() as u8 + nontensor_part_is.len() as u8)
-                    .map(|x| sv![x])
+                    .map(|x| tu8v![x])
                     .collect(),
                 int_sizes: broadcasted_shape
                     .iter()
