@@ -5,7 +5,7 @@ use std::{
     fmt::{self, Debug},
     hash::Hash,
     iter::zip,
-    ops::{Deref, DerefMut},
+    ops::{BitOr, Deref, DerefMut},
     sync::Arc,
 };
 
@@ -162,6 +162,16 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
     // ==== default only section ===
     // FUNCTIONS BELOW HERE *shouldn't* be overridden by implementors!
     // (if you do implement, this won't be picked up on by CircuitNodeDefer (union types etc)!)
+    #[inline]
+    fn compute_flags_default(&self) -> CircuitFlags {
+        let mut result: CircuitFlags = CircuitFlags::IS_EXPLICITLY_COMPUTABLE
+            | CircuitFlags::IS_CONSTANT
+            | CircuitFlags::CAN_BE_SAMPLED;
+        for child in self.children() {
+            result &= child.info().flags;
+        }
+        result
+    }
 
     fn hash_base16(&self) -> String {
         base16::encode_lower(&self.info().hash)
@@ -704,6 +714,7 @@ macro_rules! define_circuit_union_impl {
         }
 
         impl rr_util::eq_by_big_hash::EqByBigHash for $name {
+            #[inline]
             fn hash(&self) -> rr_util::util::HashBytes {
                 self.info().hash
             }
@@ -1086,6 +1097,7 @@ macro_rules! circuit_node_extra_impl {
             }
         }
         impl rr_util::eq_by_big_hash::EqByBigHash for $type_name {
+            #[inline]
             fn hash(&self) -> rr_util::util::HashBytes {
                 self.info().hash
             }
@@ -1112,9 +1124,11 @@ macro_rules! circuit_node_extra_impl {
 #[macro_export]
 macro_rules! circuit_node_auto_impl {
     ($the_uuid:literal) => {
+        #[inline]
         fn info(&self) -> &$crate::CachedCircuitInfo {
             &self.info
         }
+        #[inline]
         fn name(&self) -> Option<&str> {
             self.name.as_deref()
         }
@@ -1206,15 +1220,57 @@ impl<T: CircuitNodeDefer> CircuitNode for T {
 #[derive(Clone, Default)]
 pub struct CachedCircuitInfo {
     pub shape: Shape,
-    pub is_constant: bool,
-    pub is_explicitly_computable: bool,
-    pub can_be_sampled: bool,
+    pub flags: CircuitFlags,
     pub hash: HashBytes,
     pub device_dtype: TorchDeviceDtypeOp,
     pub named_axes: NamedAxes,
     pub symbolic_size_constraints: SymbolicSizeConstraints,
 }
 
+#[derive(Clone, Copy, Hash, Debug, Eq, PartialEq, Default)] // note default is all false, meant to be overwritten
+pub struct CircuitFlags(pub u8);
+impl CircuitFlags {
+    pub const IS_CONSTANT: CircuitFlags = CircuitFlags(0b001);
+    pub const IS_EXPLICITLY_COMPUTABLE: CircuitFlags = CircuitFlags(0b010);
+    pub const CAN_BE_SAMPLED: CircuitFlags = CircuitFlags(0b100);
+
+    pub const NONE: CircuitFlags = CircuitFlags(0b0);
+    pub fn check(self, other: CircuitFlags) -> bool {
+        (self & other).0 != 0
+    }
+}
+impl BitOr for CircuitFlags {
+    type Output = CircuitFlags;
+    fn bitor(self, rhs: Self) -> Self::Output {
+        Self(self.0 | rhs.0)
+    }
+}
+use std::ops::{BitAnd, Not};
+impl BitAnd for CircuitFlags {
+    type Output = CircuitFlags;
+    fn bitand(self, rhs: Self) -> Self::Output {
+        Self(self.0 & rhs.0)
+    }
+}
+use std::ops::BitOrAssign;
+impl BitOrAssign for CircuitFlags {
+    fn bitor_assign(&mut self, rhs: Self) {
+        self.0 |= rhs.0
+    }
+}
+use std::ops::BitAndAssign;
+impl BitAndAssign for CircuitFlags {
+    fn bitand_assign(&mut self, rhs: Self) {
+        self.0 &= rhs.0
+    }
+}
+impl Not for CircuitFlags {
+    type Output = CircuitFlags;
+    fn not(self) -> CircuitFlags {
+        Self(!self.0)
+    }
+}
+
 /// don't want to print hash with Debug, for now just print shape
 impl Debug for CachedCircuitInfo {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
@@ -1243,6 +1299,15 @@ impl CachedCircuitInfo {
         hash_prefix.copy_from_slice(&self.hash[..8]);
         usize::from_le_bytes(hash_prefix)
     }
+    pub fn is_constant(&self) -> bool {
+        self.flags.check(CircuitFlags::IS_CONSTANT)
+    }
+    pub fn can_be_sampled(&self) -> bool {
+        self.flags.check(CircuitFlags::CAN_BE_SAMPLED)
+    }
+    pub fn is_explicitly_computable(&self) -> bool {
+        self.flags.check(CircuitFlags::IS_EXPLICITLY_COMPUTABLE)
+    }
 }
 
 #[apply(python_error_exception)]
@@ -1473,12 +1538,12 @@ pub fn evaluate(circ: CircuitRc) -> Result<Tensor> {
 
 #[pyfunction]
 pub fn check_evaluable(circuit: CircuitRc) -> Result<()> {
-    if !circuit.info().is_explicitly_computable {
+    if !circuit.info().is_explicitly_computable() {
         bail!(TensorEvalError::NotExplicitlyComputable {
             circuit: circuit.clone()
         })
     }
-    if !circuit.info().is_constant {
+    if !circuit.info().is_constant() {
         bail!(TensorEvalError::NotConstant {
             circuit: circuit.clone()
         })
@@ -1939,17 +2004,17 @@ impl PyCircuitBase {
 
     #[getter]
     fn is_constant(&self) -> bool {
-        self.info().is_constant
+        self.info().is_constant()
     }
 
     #[getter]
     fn is_explicitly_computable(&self) -> bool {
-        self.info().is_explicitly_computable
+        self.info().is_explicitly_computable()
     }
 
     #[getter]
     fn can_be_sampled(&self) -> bool {
-        self.info().can_be_sampled
+        self.info().can_be_sampled()
     }
 
     #[getter]
