@@ -350,7 +350,7 @@ impl ModuleSpec {
         Ok(out)
     }
 
-    fn check_all_args_present(&self) -> Result<()> {
+    pub fn check_all_args_present(&self) -> Result<()> {
         let missing_symbols: HashSet<_> = self
             .arg_specs
             .iter()
@@ -365,7 +365,7 @@ impl ModuleSpec {
         Ok(())
     }
 
-    fn check_unique_arg_names(&self) -> Result<()> {
+    pub fn check_unique_arg_names(&self) -> Result<()> {
         // TODO: maybe cache me (as needed)!
         if self.arg_specs.iter().any(|x| x.symbol.name().is_none()) {
             bail!(ConstructError::ModuleSomeArgsNamedNone {
