@@ -1122,9 +1122,9 @@ pub fn circuit_to_size_color_code(circuit: CircuitRc) -> CliColor {
 }
 
 pub fn circuit_to_computability_color_code(circuit: CircuitRc) -> CliColor {
-    if !circuit.info().can_be_sampled {
+    if !circuit.info().can_be_sampled() {
         clicolor!(Green)
-    } else if !circuit.info().is_explicitly_computable {
+    } else if !circuit.info().is_explicitly_computable() {
         clicolor!(Yellow)
     } else {
         CliColor::NONE
