@@ -12,8 +12,8 @@ use crate::{
     circuit_node_auto_impl, circuit_node_extra_impl,
     circuit_node_private::{CircuitNodeComputeInfoImpl, CircuitNodeHashItems},
     circuit_utils::{child_name_with_maybe_paren, OperatorPriority},
-    new_rc_unwrap, CachedCircuitInfo, CircuitNode, CircuitNodeAutoName, CircuitRc, PyCircuitBase,
-    TensorEvalError,
+    new_rc_unwrap, CachedCircuitInfo, CircuitFlags, CircuitNode, CircuitNodeAutoName, CircuitRc,
+    PyCircuitBase, TensorEvalError,
 };
 
 #[pyclass(extends=PyCircuitBase)]
@@ -100,8 +100,8 @@ impl CircuitNodeComputeInfoImpl for SetSymbolicShape {
         Ok(out)
     }
 
-    fn compute_is_explicitly_computable(&self) -> bool {
-        false
+    fn compute_flags(&self) -> CircuitFlags {
+        self.node.compute_flags_default() & !CircuitFlags::IS_EXPLICITLY_COMPUTABLE
     }
 }
 
