@@ -14,8 +14,8 @@ use crate::{
     circuit_node_auto_impl, circuit_node_extra_impl,
     circuit_node_private::{CircuitNodeComputeInfoImpl, CircuitNodeHashItems},
     circuit_utils::{child_name_with_maybe_paren, OperatorPriority},
-    new_rc, new_rc_unwrap, CachedCircuitInfo, CircuitNode, CircuitNodeAutoName, CircuitRc,
-    ConstructError, PyCircuitBase, Scalar, TensorEvalError,
+    new_rc, new_rc_unwrap, CachedCircuitInfo, CircuitFlags, CircuitNode, CircuitNodeAutoName,
+    CircuitRc, ConstructError, PyCircuitBase, Scalar, TensorEvalError,
 };
 
 /// Tags a Circuit with a UUID. Use this to make two otherwise equal nodes distinct.
@@ -198,14 +198,8 @@ impl CircuitNodeComputeInfoImpl for DiscreteVar {
         self.values.info().shape[1..].iter().cloned().collect()
     }
 
-    fn compute_can_be_sampled(&self) -> bool {
-        true
-    }
-    fn compute_is_constant(&self) -> bool {
-        false
-    }
-    fn compute_is_explicitly_computable(&self) -> bool {
-        false
+    fn compute_flags(&self) -> CircuitFlags {
+        CircuitFlags::CAN_BE_SAMPLED
     }
 }
 
@@ -371,14 +365,11 @@ impl CircuitNodeComputeInfoImpl for StoredCumulantVar {
         self.cumulants[&1].info().shape.clone()
     }
 
-    fn compute_can_be_sampled(&self) -> bool {
-        self.cumulants.keys().max().unwrap() <= &2
-    }
-    fn compute_is_constant(&self) -> bool {
-        false
-    }
-    fn compute_is_explicitly_computable(&self) -> bool {
-        false
+    fn compute_flags(&self) -> CircuitFlags {
+        if self.cumulants.keys().max().unwrap() <= &2 {
+            return CircuitFlags::CAN_BE_SAMPLED;
+        }
+        CircuitFlags::NONE
     }
 }
 
