@@ -23,6 +23,7 @@ use rr_util::{
     tensor_util::{
         check_canon_idxs, compose, IndexTensor, Shape, Slice, TensorAxisIndex, TensorIndex, USlice,
     },
+    tu8v,
     union_find::UnionFind,
     unwrap,
     util::{
@@ -303,7 +304,7 @@ pub fn einsum_nest_path(einsum: &Einsum, path: Vec<Vec<usize>>) -> Einsum {
                     .intersection(&outer_indices_set)
                     .copied()
                     .collect();
-                outer_indices.sort();
+                outer_indices.as_mut_slice().sort();
                 outer_indices
             }
         };
@@ -442,13 +443,13 @@ pub fn make_broadcast(node: &CircuitRc, out_shape: &Shape) -> Result<CircuitRc>
         .enumerate()
         .map(|(i, x)| {
             if *x == 1 {
-                sv![]
+                tu8v![]
             } else {
-                sv![(i + rank_dif) as u8]
+                tu8v![(i + rank_dif) as u8]
             }
         })
         .collect();
-    let output_ints: RInts = (0..(out_shape.len())).map(|i| sv![i as u8]).collect();
+    let output_ints: RInts = (0..(out_shape.len())).map(|i| tu8v![i as u8]).collect();
     let int_sizes = out_shape.clone();
     Ok(Rearrange::nrc_elim_identity(
         node.clone(),
@@ -649,7 +650,7 @@ pub fn generalfunction_pull_removable_axes(node: &GeneralFunction) -> Option<Cir
     let removable_axes_inp = get_removable_axes(&node.nodes[0]);
     let removable_batchable_inp: HashSet<usize> = removable_axes_inp
         .iter()
-        .filter(|x| **x < node.nodes[0].info().rank() - node.num_non_batchable_output_dims)
+        .filter(|x| **x < node.nodes[0].info().rank() - node.num_non_batchable_output_dims as usize)
         .cloned()
         .collect();
     if removable_batchable_inp.is_empty() {
@@ -728,7 +729,8 @@ pub fn einsum_pull_removable_axes(einsum: &Einsum) -> Option<CircuitRc> {
     }
     let ints_in_new_args: HashSet<u8> = new_args
         .iter()
-        .flat_map(|(_node, ints)| ints.clone())
+        .flat_map(|(_node, ints)| ints.as_slice())
+        .cloned()
         .collect();
 
     // multiply by product of removed reduced axes
@@ -740,7 +742,7 @@ pub fn einsum_pull_removable_axes(einsum: &Einsum) -> Option<CircuitRc> {
         .map(|i| shape_map[i] as f64)
         .product();
     if scalar_mul != 1.0 {
-        new_args.push((Scalar::new(scalar_mul, sv![], None).rc(), sv![]));
+        new_args.push((Scalar::new(scalar_mul, sv![], None).rc(), tu8v![]));
     }
 
     let new_out_ints: EinsumAxes = einsum
@@ -817,9 +819,9 @@ pub fn add_pull_removable_axes(add: &Add, remove_non_common_axes: bool) -> Optio
     let post_rearrange_spec = RearrangeSpec::new_canon(
         (0..add.info().rank())
             .filter(|x| !intersection.contains(x))
-            .map(|x| sv![x as u8])
+            .map(|x| tu8v![x as u8])
             .collect(),
-        (0..add.info().rank()).map(|x| sv![x as u8]).collect(),
+        (0..add.info().rank()).map(|x| tu8v![x as u8]).collect(),
         shape_to_op_shape(&add.info().shape),
     );
     let new_operands = zip(add.nodes_and_rank_differences(), removable_axes)
@@ -846,10 +848,10 @@ pub fn add_pull_removable_axes(add: &Add, remove_non_common_axes: bool) -> Optio
                 for i in 0..node.info().rank() {
                     if removable_here.contains(&i) {
                         if started && !intersection.contains(&(i + rank_difference)) {
-                            output_ints.push(sv![]);
+                            output_ints.push(tu8v![]);
                         }
                     } else {
-                        output_ints.push(sv![count]);
+                        output_ints.push(tu8v![count]);
                         count += 1;
                         started = true;
                     }
@@ -857,7 +859,7 @@ pub fn add_pull_removable_axes(add: &Add, remove_non_common_axes: bool) -> Optio
 
                 let broadcast_rspec = RearrangeSpec::new_canon(
                     (0..raw_removed.info().rank())
-                        .map(|i| sv![i as u8])
+                        .map(|i| tu8v![i as u8])
                         .collect(),
                     output_ints,
                     shape_to_op_shape(&raw_removed.info().shape),
@@ -1076,7 +1078,7 @@ pub fn einsum_merge_scalars(einsum: &Einsum) -> Option<Einsum> {
         return None;
     }
     if scalar_mul != 1.0 {
-        new_args.push((Scalar::new(scalar_mul, sv![], None).rc(), sv![]))
+        new_args.push((Scalar::new(scalar_mul, sv![], None).rc(), tu8v![]))
     }
     Some(Einsum::new(new_args, einsum.out_axes.clone(), None))
 }
@@ -1400,7 +1402,9 @@ pub fn push_down_index_raw(
             }
         }
         Circuit::GeneralFunction(inner) => {
-            if !inner.is_batchable() || inner.num_non_batchable_output_dims == inner.info().rank() {
+            if !inner.is_batchable()
+                || inner.num_non_batchable_output_dims as usize == inner.info().rank()
+            {
                 bail!(PushDownIndexError::NoopOnGeneralFunction {
                     index_node: node.clone(),
                     is_batchable: inner.is_batchable(),
@@ -1408,10 +1412,11 @@ pub fn push_down_index_raw(
                     inner_rank: inner.rank()
                 })
             } else {
-                let rank_to_pass = inner.info().rank() - inner.num_non_batchable_output_dims;
-                let new_operands = zip(&inner.nodes, &inner.input_batchability)
+                let rank_to_pass =
+                    inner.info().rank() - inner.num_non_batchable_output_dims as usize;
+                let new_operands = zip(&inner.nodes, inner.input_batchability)
                     .map(|(child, batchable)| {
-                        if *batchable {
+                        if batchable {
                             let ident_rank = child.info().rank() - rank_to_pass;
                             let index_passed = TensorIndex(
                                 node.index.0[..rank_to_pass]
@@ -1710,15 +1715,17 @@ pub fn concat_repeat_to_rearrange(concat: &Concat) -> Option<Concat> {
                 new_nodes.push(Rearrange::nrc_elim_identity(
                     prev_node.clone(),
                     RearrangeSpec::new(
-                        (0..prev_node.info().rank()).map(|x| sv![x as u8]).collect(),
+                        (0..prev_node.info().rank())
+                            .map(|x| tu8v![x as u8])
+                            .collect(),
                         (0..prev_node.info().rank())
                             .map(|x| {
                                 if x == concat.axis {
                                     // create group which is (count, axis_size) for total dim size of
                                     // count * axis_size()
-                                    sv![prev_node.info().rank() as u8, x as u8]
+                                    tu8v![prev_node.info().rank() as u8, x as u8]
                                 } else {
-                                    sv![x as u8]
+                                    tu8v![x as u8]
                                 }
                             })
                             .collect(),
@@ -1931,7 +1938,7 @@ pub fn add_fuse_scalar_multiples(add: &Add) -> Option<Add> {
                 let name = einsum.name_cloned();
                 let out_axes = einsum.out_axes;
                 let mut v = einsum.args;
-                v.push((Scalar::new(scalar, sv![], scalar_name).rc(), sv![]));
+                v.push((Scalar::new(scalar, sv![], scalar_name).rc(), tu8v![]));
                 Einsum::nrc(v, out_axes, name)
             }
         })
@@ -1951,8 +1958,8 @@ pub fn einsum_permute_to_rearrange(einsum: &Einsum) -> Option<Rearrange> {
         return Some(Rearrange::new(
             einsum.args[0].0.clone(),
             RearrangeSpec::new(
-                einsum.args[0].1.iter().map(|z| sv![*z]).collect(),
-                einsum.out_axes.iter().map(|z| sv![*z]).collect(),
+                einsum.args[0].1.iter().map(|z| tu8v![*z]).collect(),
+                einsum.out_axes.iter().map(|z| tu8v![*z]).collect(),
                 sv![OpSize::NONE;(*einsum.out_axes.iter().max().unwrap()+1) as usize],
             )
             .unwrap()
@@ -1978,14 +1985,14 @@ pub enum PushDownIndexError {
     NoopOnGeneralFunction {
         index_node: Index,
         is_batchable: bool,
-        num_non_batch_out: usize,
+        num_non_batch_out: u8,
         inner_rank: usize,
     },
     #[error("index_node={index_node:?} is_batchable={is_batchable} num_non_batch_out={num_non_batch_out} inner_rank={inner_rank} top_axis_indices={top_axis_indices:?} ({e_name})")]
     GeneralFunctionSomeAxesNotPossible {
         index_node: Index,
         is_batchable: bool,
-        num_non_batch_out: usize,
+        num_non_batch_out: u8,
         inner_rank: usize,
         top_axis_indices: Vec<TensorAxisIndex>,
     },
