@@ -236,11 +236,10 @@ pub fn optimize_circuit_py(
     circuit: CircuitRc,
     settings: OptimizationSettings,
 ) -> Result<CircuitRc> {
-    let mut context = OptimizationContext {
-        settings,
-        ..Default::default()
-    };
-    optimize_circuit(circuit, &mut context)
+    optimize_circuit(
+        circuit.clone(),
+        &mut OptimizationContext::new_settings_circuit(settings, circuit),
+    )
 }
 
 pub fn optimize_circuit(
@@ -389,7 +388,7 @@ pub fn optimize_and_evaluate_many(
     settings: OptimizationSettings,
 ) -> Result<Vec<Tensor>> {
     for c in &circuits {
-        if !c.info().is_explicitly_computable {
+        if !c.info().is_explicitly_computable() {
             bail!(SchedulingError::NotExplicitlyComputable { circuit: c.clone() });
         }
     }
