@@ -161,7 +161,8 @@ pub fn add_pull_concat(add: &Add) -> Option<CircuitRc> {
 // pub fn rearrange_pull_concat(rearrange: &Rearrange) -> Option<Concat> {}
 #[pyfunction]
 pub fn generalfunction_pull_concat(generalfunction: &GeneralFunction) -> Option<Concat> {
-    let batch_rank = generalfunction.info().rank() - generalfunction.num_non_batchable_output_dims;
+    let batch_rank =
+        generalfunction.info().rank() - generalfunction.num_non_batchable_output_dims as usize;
     for operand in &generalfunction.nodes {
         if let Circuit::Concat(concat) = &***operand &&  concat.axis < batch_rank {
             let mut cur = 0;
@@ -172,7 +173,7 @@ pub fn generalfunction_pull_concat(generalfunction: &GeneralFunction) -> Option<
                     let span_here = cnode.info().shape[concat.axis];
                     let new_operands = zip(
                         &generalfunction.nodes,
-                        &generalfunction.input_batchability,
+                        generalfunction.input_batchability,
                     )
                     .map(|(node, batchability)| {
                         if !batchability {
