@@ -1,6 +1,6 @@
 use std::fmt::{self, Display};
 
-use anyhow::{bail, Context, Result};
+use anyhow::{anyhow, bail, Context, Result};
 use circuit_base::{
     cached_circuit_properties::max_non_leaf_size,
     circuit_utils::{hash_to_node_non_free, toposort_circuit},
@@ -8,7 +8,7 @@ use circuit_base::{
     module::{any_children_with_symbolic_sizes, substitute_all_modules},
     prelude::*,
     print::{oom_fmt, PrintOptions},
-    visit_circuit_non_free, Add, Array, IrreducibleNode, Leaf, ModuleSpec, Scalar, Symbol,
+    visit_circuit_non_free, Array, IrreducibleNode, Leaf, ModuleSpec, Scalar, Symbol,
 };
 use itertools::Itertools;
 use macro_rules_attribute::apply;
@@ -31,6 +31,7 @@ use thiserror::Error;
 
 use crate::{
     circuit_optimizer::{OptimizationContext, OptimizationSettings},
+    module_rewrite::deep_module_remove_unused_inputs,
     schedule_send::ScheduleToSend,
     scheduling_alg::{Dag, DagSimpSettings},
 };
@@ -76,6 +77,49 @@ pub struct Schedule {
 
 #[pymethods]
 impl Schedule {
+    pub fn validate(&self, validate_output: bool) -> Result<()> {
+        // NOTE: values can't be computed or dropped multiple times atm.
+        let mut prior_compute_numbers = HashSet::default();
+        let mut prior_drop_numbers = HashSet::default();
+
+        for num in self.scalars.keys() {
+            if self.constants.contains_key(num) {
+                bail!("num={num} used for both constants and scalars!");
+            }
+        }
+        for instruction in &self.instructions {
+            match instruction {
+                Instruction::Drop(drop) => {
+                    if !prior_compute_numbers.contains(drop) {
+                        bail!("drop={drop} not previously computed");
+                    }
+                    if !prior_drop_numbers.insert(drop) {
+                        bail!("drop={drop} was already dropped!");
+                    }
+                }
+                Instruction::Compute(key, circuit) => {
+                    if !prior_compute_numbers.insert(key) {
+                        bail!("key={key} was already computed!");
+                    }
+                    if circuit.is_leaf() {
+                        bail!("leaf circuit is being computed circuit={circuit:?}");
+                    }
+                }
+            }
+        }
+
+        if validate_output {
+            let output_id = self.output_circuit.as_ref().ok_or(anyhow!("no output!"))?.0;
+            for computed in prior_compute_numbers {
+                if !prior_drop_numbers.contains(computed) && *computed != output_id {
+                    bail!("computed={computed} is never dropped (or returned)");
+                }
+            }
+        }
+
+        Ok(())
+    }
+
     pub fn replace_tensors(&self, map: HashMap<HashBytes, Tensor>) -> PyResult<Self> {
         let mut result = self.clone();
         for (k, v) in map {
@@ -282,7 +326,7 @@ pub struct ScheduleStats {
     max_mem: BigUint,
     #[pyo3(get)]
     constant_mem: BigUint, // this has already been allocated so it can't be over 2^64
-    // #[pyo3(get)]
+    #[pyo3(get)]
     max_circuit_set: HashSet<CircuitRc>,
 }
 
@@ -356,17 +400,22 @@ fn child_from_key(key: usize, shape: Shape) -> Symbol {
     Symbol::new_with_random_uuid(shape, Some(key.to_string()))
 }
 
-fn identity(key: usize, shape: Shape) -> CircuitRc {
-    Add::nrc(
-        vec![Symbol::new_with_random_uuid(shape, Some(key.to_string())).rc()],
-        None,
-    )
-}
-
 pub fn get_children_keys(circuit: CircuitRc) -> Vec<usize> {
     circuit.children().map(get_child_key).collect()
 }
 
+/// see https://pyo3.rs/main/doc/pyo3/marker/struct.python#method.new_pool
+pub unsafe fn with_gil_pool<F, R>(f: F) -> R
+where
+    F: for<'py> FnOnce(Python<'py>) -> R,
+{
+    Python::with_gil(|py| {
+        let pool = unsafe { py.new_pool() };
+        let py = pool.python();
+        f(py)
+    })
+}
+
 pub fn evaluate_schedule(schedule: &Schedule) -> HashMap<usize, Tensor> {
     let mut live: HashMap<usize, Tensor> = schedule
         .constants
@@ -381,7 +430,7 @@ pub fn evaluate_schedule(schedule: &Schedule) -> HashMap<usize, Tensor> {
             .map(|x| (*x.0, x.1.eval_tensors(&[], &schedule.device_dtype).unwrap())),
     );
     for s in &schedule.instructions {
-        match s {
+        let run = |_py: Python| match s {
             Instruction::Compute(key, circ) => {
                 let child_keys: Vec<usize> = circ.children().map(get_child_key).collect();
                 child_keys.iter().for_each(|child_key| {
@@ -405,7 +454,8 @@ pub fn evaluate_schedule(schedule: &Schedule) -> HashMap<usize, Tensor> {
             Instruction::Drop(key) => {
                 live.remove(key);
             }
-        }
+        };
+        unsafe { with_gil_pool(run) }
     }
     live
 }
@@ -486,7 +536,7 @@ pub fn evaluate_schedule_adjust_numerical_scale(
         })
     }));
     for s in &schedule.instructions {
-        match s {
+        let run = |_py: Python| match s {
             Instruction::Compute(key, circ) => {
                 circ.children().for_each(|x: CircuitRc| {
                     if !live.contains_key(&get_child_key(x)) {
@@ -555,7 +605,8 @@ pub fn evaluate_schedule_adjust_numerical_scale(
             Instruction::Drop(hash) => {
                 live.remove(hash);
             }
-        }
+        };
+        unsafe { with_gil_pool(run) }
     }
     live.iter()
         .map(|(k, v)| (*k, set_scale(v, 1.0).0))
@@ -600,7 +651,7 @@ pub fn order_to_schedule(
 
     let mut seen_dependencies: HashSet<usize> = HashSet::default();
     for ex in order.iter().rev() {
-        for child in ex.children() {
+        for child in ex.non_free_children() {
             let next_id = circ_to_id.len();
             let dep = *circ_to_id.entry(child.info().hash).or_insert(next_id);
             if !Leaf::matches(&child)
@@ -688,6 +739,10 @@ pub fn circuit_to_schedule(
     circuit: CircuitRc,
     context: &mut OptimizationContext,
 ) -> Result<Schedule> {
+    // we could avoid this if we wanted, but a bit annoying
+    // (requires handle rearrange due to unused arg + empty instruction case!)
+    let circuit =
+        deep_module_remove_unused_inputs(circuit, &mut context.cache.module_removed_unused, true);
     if is_python_running() && any_children_with_symbolic_sizes(circuit.clone()) {
         Python::with_gil(|py| {
             pyo3::PyErr::warn(
@@ -768,20 +823,36 @@ pub fn circuit_to_schedule(
     );
     out.output_circuit = Some((kept_keys[0], circuit));
     let out = replace_module_schedules(&out, context);
+    out.validate(true).expect("should be valid");
     Ok(out)
 }
 
 pub fn circuit_to_schedule_naive_toposort(circuit: CircuitRc) -> Schedule {
     let circuit = substitute_all_modules(circuit);
     let toposorted = toposort_circuit(circuit.clone());
+    let mut order = Vec::new();
+    let mut scalars = Vec::new();
+    let mut constants = Vec::new();
+
+    for c in toposorted {
+        if let Some(sc) = c.as_scalar() {
+            scalars.push(sc.clone())
+        } else if let Some(constant) = Option::<IrreducibleNode>::from(c.clone().c()) {
+            constants.push(constant)
+        } else {
+            order.push(c)
+        }
+    }
+
     let (mut result, kept_keys) = order_to_schedule(
-        &toposorted,
-        &vec![],
-        &vec![],
+        &order,
+        &constants,
+        &scalars,
         HashSet::from_iter([circuit.info().hash]),
         get_compatible_dtype(&circuit),
     );
     result.output_circuit = Some((kept_keys[0], circuit));
+    result.validate(true).expect("should be valid");
     result
 }
 
@@ -833,85 +904,87 @@ fn replace_module_schedules(schedule: &Schedule, context: &mut OptimizationConte
             Instruction::Drop(_d) => result.instructions.push(ins.clone()),
             Instruction::Compute(outermost_k, v) => match &***v {
                 Circuit::Module(mn) => {
+                    mn.spec
+                        .check_all_args_present()
+                        .expect("unused args removed at start of circuit_to_schedule!");
+
                     let original_children: Vec<usize> =
                         mn.nodes.iter().cloned().map(get_child_key).collect();
 
                     let inner_schedule = schedule_module_spec(&mn.spec, context);
-                    if inner_schedule.instructions.is_empty() {
-                        result.instructions.push(Instruction::Compute(
-                            *outermost_k,
-                            identity(
-                                get_child_key(mn.nodes[0].clone()),
-                                mn.nodes[0].info().shape.clone(),
-                            ),
-                        ));
-                    } else {
-                        let mut inner_to_outer_key: HashMap<usize, usize> = HashMap::from_iter([(
+                    let mut inner_to_outer_key = HashMap::default();
+                    for (inner_k, sc) in &inner_schedule.scalars {
+                        let outer = scalar_to_id.get(sc).cloned().unwrap_or_else(|| {
+                            let this_key = next_key;
+                            scalar_to_id.insert(sc.clone(), this_key);
+                            result.scalars.insert(this_key, sc.clone());
+                            next_key += 1;
+                            this_key
+                        });
+                        inner_to_outer_key.try_insert(*inner_k, outer).unwrap();
+                    }
+                    for (inner_k, irreducible) in &inner_schedule.constants {
+                        let outer =
+                            irreducible_to_id
+                                .get(irreducible)
+                                .cloned()
+                                .unwrap_or_else(|| {
+                                    if let Some(pos) =
+                                        mn.spec.arg_specs.iter().position(|argspec| {
+                                            argspec.symbol.info().hash == irreducible.info().hash
+                                        })
+                                    {
+                                        original_children[pos]
+                                    } else {
+                                        let this_key = next_key;
+                                        irreducible_to_id.insert(irreducible.clone(), this_key);
+                                        result.constants.insert(this_key, irreducible.clone());
+                                        next_key += 1;
+                                        this_key
+                                    }
+                                });
+                        inner_to_outer_key.try_insert(*inner_k, outer).unwrap();
+                    }
+
+                    assert!(
+                        !inner_schedule.instructions.is_empty(),
+                        "empty_modules removed at start of circuit to schedule"
+                    );
+
+                    inner_to_outer_key
+                        .try_insert(
                             inner_schedule.output_circuit.clone().unwrap().0,
                             *outermost_k,
-                        )]);
-                        for (inner_k, sc) in &inner_schedule.scalars {
-                            let outer = scalar_to_id.get(sc).cloned().unwrap_or_else(|| {
-                                scalar_to_id.insert(sc.clone(), next_key);
-                                result.scalars.insert(next_key, sc.clone());
-                                next_key += 1;
-                                next_key - 1
-                            });
-                            inner_to_outer_key.try_insert(*inner_k, outer).ok();
-                        }
-                        for (inner_k, irreducible) in &inner_schedule.constants {
-                            let outer =
-                                irreducible_to_id
-                                    .get(irreducible)
-                                    .cloned()
-                                    .unwrap_or_else(|| {
-                                        if let Some(pos) =
-                                            mn.spec.arg_specs.iter().position(|argspec| {
-                                                argspec.symbol.info().hash
-                                                    == irreducible.info().hash
-                                            })
-                                        {
-                                            original_children[pos]
-                                        } else {
-                                            irreducible_to_id.insert(irreducible.clone(), next_key);
-                                            result.constants.insert(next_key, irreducible.clone());
-                                            next_key += 1;
-                                            next_key - 1
-                                        }
-                                    });
-                            inner_to_outer_key.try_insert(*inner_k, outer).ok();
-                        }
-                        for inner_ins in &inner_schedule.instructions {
-                            match inner_ins {
-                                Instruction::Drop(drop) => {
-                                    result
-                                        .instructions
-                                        .push(Instruction::Drop(inner_to_outer_key[drop]));
-                                }
-                                Instruction::Compute(inner_k, c) => {
-                                    let new_c = c
-                                        .map_children_unwrap(|child| {
-                                            let child_inner_k = get_child_key(child.clone());
-                                            let child_outer_k =
-                                                inner_to_outer_key.get(&child_inner_k).unwrap();
-                                            child_from_key(
-                                                *child_outer_k,
-                                                child.info().shape.clone(),
-                                            )
+                        )
+                        .unwrap();
+
+                    for inner_ins in &inner_schedule.instructions {
+                        match inner_ins {
+                            Instruction::Drop(drop) => {
+                                result
+                                    .instructions
+                                    .push(Instruction::Drop(inner_to_outer_key[drop]));
+                            }
+                            Instruction::Compute(inner_k, c) => {
+                                assert!(!c.is_module());
+                                let new_c = c
+                                    .map_children_unwrap(|child| {
+                                        let child_inner_k = get_child_key(child.clone());
+                                        let child_outer_k =
+                                            inner_to_outer_key.get(&child_inner_k).unwrap();
+                                        child_from_key(*child_outer_k, child.info().shape.clone())
                                             .rc()
-                                        })
-                                        .rc();
-                                    let key = inner_to_outer_key
-                                        .get(inner_k)
-                                        .cloned()
-                                        .unwrap_or_else(|| {
-                                            inner_to_outer_key.insert(*inner_k, next_key);
-                                            next_key += 1;
-                                            next_key - 1
-                                        });
-                                    result.instructions.push(Instruction::Compute(key, new_c));
-                                    inner_to_outer_key.insert(*inner_k, key);
-                                }
+                                    })
+                                    .rc();
+                                let key =
+                                    inner_to_outer_key.get(inner_k).cloned().unwrap_or_else(|| {
+                                        let this_key = next_key;
+                                        inner_to_outer_key.insert(*inner_k, this_key);
+                                        next_key += 1;
+                                        this_key
+                                    });
+                                result.instructions.push(Instruction::Compute(key, new_c));
+                                inner_to_outer_key.insert(*inner_k, key);
                             }
                         }
                     }
@@ -923,6 +996,7 @@ fn replace_module_schedules(schedule: &Schedule, context: &mut OptimizationConte
             },
         }
     }
+    result.validate(result.output_circuit.is_some()).unwrap();
 
     result
 }
