@@ -8,7 +8,7 @@ use std::{
     vec,
 };
 
-use anyhow::{bail, Result};
+use anyhow::{bail, Context, Result};
 use circuit_base::{visit_circuit, CircuitNode, CircuitNodeUnion, CircuitRc, CircuitType};
 use pyo3::{prelude::*, AsPyPointer};
 use regex::{Captures, Regex};
@@ -350,9 +350,15 @@ impl Matcher {
             MatcherData::EqM(circs) => circs.contains(&circuit),
             MatcherData::AnyFound(any_found) => any_found.lock().unwrap().are_any_found(circuit)?,
             MatcherData::Raw(f) => f.0(circuit)?,
-            MatcherData::PyFunc(pyfunc) => {
-                Python::with_gil(|py| pyfunc.call1(py, (circuit,)).and_then(|r| r.extract(py)))?
-            }
+            MatcherData::PyFunc(pyfunc) => Python::with_gil(|py| {
+                pyfunc
+                    .call1(py, (circuit,))
+                    .context("calling python matcher failed")
+                    .and_then(|r| {
+                        r.extract(py)
+                            .context("extracting from python matcher failed")
+                    })
+            })?,
             MatcherData::Not(m) => !m.call(circuit)?,
             MatcherData::Any(ms) => {
                 for m in ms {
