@@ -1,14 +1,20 @@
-use std::{fmt::Write, iter::zip};
+use std::{
+    fmt::Write,
+    iter::{once, zip},
+};
 
 use anyhow::Result;
 use circuit_base::{
     clicolor,
+    parsing::Parser,
     print::{color, CliColor, PrintOptions},
     CircuitNode, CircuitNodeSelfOnlyHash, CircuitRc,
 };
+use itertools::Itertools;
 use once_cell::unsync::Lazy;
 use pyo3::prelude::*;
 use rr_util::util::{indent, HashBytes};
+use rustc_hash::FxHashMap as HashMap;
 use uuid::uuid;
 #[pyfunction(
     hash_child_count = "true",
@@ -71,6 +77,8 @@ pub fn diff_circuits(
     let mut options = options;
     options.bijection = false;
     let mut result = "".to_owned();
+    let mut seen_diffs: HashMap<(CircuitRc, CircuitRc), String> = HashMap::default();
+    let mut seen_independent: HashMap<CircuitRc, String> = HashMap::default();
     fn recurse(
         new: CircuitRc,
         old: CircuitRc,
@@ -80,15 +88,29 @@ pub fn diff_circuits(
         require_child_count_same: bool,
         require_child_shapes_same: bool,
         require_name_same: bool,
+        seen_diffs: &mut HashMap<(CircuitRc, CircuitRc), String>,
+        seen_indep: &mut HashMap<CircuitRc, String>,
     ) -> Result<()> {
+        let diffkey = (new.clone(), old.clone());
+        if let Some(id) = seen_diffs.get(&diffkey) {
+            result.push_str(&indent(id.clone(), last_child_stack.len() * 2));
+            result.push('\n');
+            return Ok(());
+        }
+        let id = seen_diffs.len();
+
         if new == old {
             let mut new_options = options.clone();
             new_options.colorer = Some(PrintOptions::fixed_color(SAME_COLOR));
-            result.push_str(&indent(
-                new_options.repr(new.clone())?,
-                last_child_stack.len() * 2,
-            ));
+            let idstr = color(&format!(" # same {}", id), SAME_COLOR);
+            let child_printed = new_options.repr(new.clone())?;
+            let child_printed =
+                once(&(child_printed.lines().next().unwrap().to_owned() + &idstr) as &str)
+                    .chain(child_printed.lines().skip(1))
+                    .join("\n");
+            result.push_str(&indent(child_printed, last_child_stack.len() * 2));
             result.push_str("\n");
+            seen_diffs.insert(diffkey.clone(), idstr);
             return Ok(());
         }
         if compute_self_hash(
@@ -115,6 +137,9 @@ pub fn diff_circuits(
                 ),
                 last_child_stack.len() * 2,
             ));
+            let idstr = color(&format!(" # changed {}", id), *SAME_SELF_COLOR);
+            result.push_str(&idstr);
+            seen_diffs.insert(diffkey.clone(), idstr);
             result.push_str("\n");
 
             assert_eq!(new.children().count(), old.children().count());
@@ -133,6 +158,8 @@ pub fn diff_circuits(
                     require_child_count_same,
                     require_child_shapes_same,
                     require_name_same,
+                    seen_diffs,
+                    seen_indep,
                 )?;
             }
             return Ok(());
@@ -164,8 +191,13 @@ pub fn diff_circuits(
         require_child_count_same,
         require_child_shapes_same,
         require_name_same,
+        &mut seen_diffs,
+        &mut seen_independent,
     )?;
     if print_legend {
+        if &result[result.len() - 1..] != "\n" {
+            result.push('\n');
+        }
         write!(
             result,
             "{} {} {} {}\n",
