@@ -35,9 +35,9 @@ pub fn distribute(
     do_broadcasts: bool,
 ) -> Result<CircuitRc> {
     #[apply(cached_lambda)]
-    #[key((einsum.info().hash, matcher.clone()), (HashBytes, IterativeMatcherRc))]
+    #[key((einsum.info().hash, traversal.clone()), (HashBytes, IterativeMatcherRc))]
     #[use_try]
-    fn distribute_rec(einsum: Einsum, matcher: IterativeMatcherRc) -> Result<CircuitRc> {
+    fn distribute_rec(einsum: Einsum, traversal: IterativeMatcherRc) -> Result<CircuitRc> {
         if operand_idx >= einsum.args.len() {
             if allow_partial_distribute {
                 return Ok(einsum.rc());
@@ -48,15 +48,15 @@ pub fn distribute(
                 })
             };
         }
-        let updated = matcher
+        let updated = traversal
             .match_iterate(einsum.args[operand_idx].0.clone())?
-            .unwrap_or_same(matcher)
+            .unwrap_or_same(traversal)
             .0;
-        let new_matcher = if let Some(new_matcher) = require_single(updated)
-            .context("distribute doesn't support matcher per child")?
+        let new_traversal = if let Some(new_traversal) = require_single(updated)
+            .context("distribute doesn't support traversal per child")?
             .0
         {
-            new_matcher
+            new_traversal
         } else {
             return Ok(einsum.rc());
         };
@@ -66,7 +66,7 @@ pub fn distribute(
             operand_idx,
             do_broadcasts,
             suffix.clone(),
-            |new_einsum| distribute_rec(new_einsum, new_matcher.clone()),
+            |new_einsum| distribute_rec(new_einsum, new_traversal.clone()),
         );
         if distributed.is_err() && allow_partial_distribute {
             return Ok(einsum.clone().rc());
