@@ -22,7 +22,7 @@ use rr_util::{
     eq_by_big_hash::EqByBigHash,
     fn_struct, impl_eq_by_big_hash, pycall, python_error_exception,
     rearrange_spec::{OpSize, RearrangeSpec, RearrangeSpecError},
-    sv,
+    sv, tu8v,
     util::{DimNumMaker, HashBytes},
 };
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
@@ -268,14 +268,14 @@ pub fn fuse_modules_impl(
                                 .clone()
                                 .chain(same_orig_batch.clone())
                                 .chain(same_non_batch.clone())
-                                .map(|i| sv![i as u8])
+                                .map(|i| tu8v![i as u8])
                                 .collect(),
                             same_outer
                                 .chain(pad_for_orig)
                                 .chain(same_orig_batch)
                                 .chain(cum)
                                 .chain(same_non_batch.clone())
-                                .map(|i| sv![i as u8])
+                                .map(|i| tu8v![i as u8])
                                 .collect(),
                             sizes,
                         )
