@@ -33,6 +33,7 @@ pub struct PyUtils {
     pub tensor_from_bytes: PyObject,
     pub assert_tensors_close: PyObject,
     pub builtins: Py<PyModule>,
+    pub gc: Py<PyModule>,
     pub optimizing_symbolic_size_warning: PyObject,
 }
 
@@ -81,6 +82,7 @@ pub static PY_UTILS: GILLazyPy<PyUtils> = GILLazyPy::new_py(|py| {
         tensor_from_bytes: get("tensor_from_bytes"),
         assert_tensors_close: get("assert_tensors_close"),
         builtins: builtins.into(),
+        gc: PyModule::import(py, "gc").unwrap().into(),
         optimizing_symbolic_size_warning: get("OptimizingSymbolicSizeWarning"),
     }
 });
@@ -118,6 +120,11 @@ macro_rules! make_py_func {
     };
 }
 
+// currently unused, but I'm leaving around because a bit nice to use for debugging etc sometimes.
+pub fn collect_garbage() {
+    Python::with_gil(|py| PY_UTILS.gc.call_method0(py, "collect")).unwrap();
+}
+
 #[apply(make_py_func)]
 #[py_ident(py)]
 pub fn assert_tensors_close(a: Tensor, b: Tensor) -> Result<()> {
