@@ -17,11 +17,13 @@ use thiserror::Error;
 use uuid::uuid;
 
 use crate::{
+    compact_data::TinyVecU8,
     make_single_many,
     py_types::{einops_repeat, PyOpAtAxes, Tensor, PY_CIRCUIT_ITEMS},
     python_error_exception, sv,
     symbolic_size::{SymbolicSizeProduct, SIZE_PROD_MATCH},
     tensor_util::{check_canon_idxs, PyParseError, Shape},
+    tu8v,
     util::{
         dict_to_list, filter_out_idx, is_unique, vec_map_insert, AxisInt, EinsumAxes, HashBytes,
         NamedAxes, ALPHABET,
@@ -102,7 +104,7 @@ pub fn shape_to_op_shape(shape: &Shape) -> OpShape {
 
 // RInnerInts distribution is heavy tailed - almost everything's 1, but there might be up to 10.
 // we use multiple of 8 bc they're u8 and we'd just pad if we used less
-pub type RInnerInts = Sv<[AxisInt; 8]>;
+pub type RInnerInts = TinyVecU8;
 pub type RInts = Sv<[RInnerInts; 4]>;
 
 #[pyclass]
@@ -212,9 +214,9 @@ impl FromStr for RearrangeSpec {
                 .map(|tl| {
                     if let Some(c) = tl.get(1) {
                         let num = parse_num(c.as_str())?;
-                        Ok(sv![handle_int(num, &mut sizes)?])
+                        Ok(tu8v![handle_int(num, &mut sizes)?])
                     } else if let Some(c) = tl.get(2) {
-                        Ok(sv![handle_string(c.as_str().to_owned(), &mut sizes)?])
+                        Ok(tu8v![handle_string(c.as_str().to_owned(), &mut sizes)?])
                     } else if let Some(c) = tl.get(3) {
                         c.as_str()
                             .split_whitespace()
@@ -280,7 +282,7 @@ impl RearrangeSpec {
     #[staticmethod]
     pub fn flatten(rank: u8) -> Self {
         Self::new(
-            (0..rank).map(|i| sv![i]).collect(),
+            (0..rank).map(|i| tu8v![i]).collect(),
             sv![(0..rank).collect()],
             sv![OpSize::from(None); rank as usize],
         )
@@ -293,7 +295,7 @@ impl RearrangeSpec {
         Self::check_rank(rank)?;
         Ok(Self::new(
             sv![(0..rank as u8).collect()],
-            (0..rank).map(|i| sv![i as u8]).collect(),
+            (0..rank).map(|i| tu8v![i as u8]).collect(),
             shape_to_op_shape(&shape),
         )
         .unwrap())
@@ -308,13 +310,13 @@ impl RearrangeSpec {
         let new_ndim = (ndim + shape.len()) - 1;
         Self::check_rank(new_ndim)?;
 
-        let mut initial_inp: RInts = (0..(ndim - 1) as u8).map(|i| sv![i]).collect();
+        let mut initial_inp: RInts = (0..(ndim - 1) as u8).map(|i| tu8v![i]).collect();
         let unflatten_ints = (ndim - 1) as u8..(shape.len() + ndim - 1) as u8;
         initial_inp.insert(axis, unflatten_ints.clone().collect());
         let out_ints: RInts = (0..axis as u8)
             .chain(unflatten_ints)
             .chain(axis as u8..(ndim - 1) as u8)
-            .map(|i| sv![i])
+            .map(|i| tu8v![i])
             .collect();
         let op_shape: OpShape = iter::repeat(OpSize::NONE)
             .take(ndim - 1)
@@ -330,8 +332,8 @@ impl RearrangeSpec {
     #[staticmethod]
     pub fn ident(rank: u8) -> Self {
         Self::new(
-            (0..rank).map(|i| sv![i]).collect(),
-            (0..rank).map(|i| sv![i]).collect(),
+            (0..rank).map(|i| tu8v![i]).collect(),
+            (0..rank).map(|i| tu8v![i]).collect(),
             sv![OpSize::from(None); rank as usize],
         )
         .unwrap()
@@ -360,10 +362,10 @@ impl RearrangeSpec {
         Self::check_rank(rank)?;
         let out = Self::new(
             (new_shape.len()..new_shape.len() + old_rank)
-                .map(|i| sv![i as u8])
+                .map(|i| tu8v![i as u8])
                 .collect(),
             (0..new_shape.len() + old_rank)
-                .map(|i| sv![i as u8])
+                .map(|i| tu8v![i as u8])
                 .collect(),
             new_shape
                 .iter()
@@ -408,8 +410,8 @@ impl RearrangeSpec {
         check_permutation(&permutation)?;
 
         let out = Self::new(
-            (0..permutation.len() as u8).map(|i| sv![i]).collect(),
-            permutation.iter().map(|i| sv![*i as u8]).collect(),
+            (0..permutation.len() as u8).map(|i| tu8v![i]).collect(),
+            permutation.iter().map(|i| tu8v![*i as u8]).collect(),
             sv![OpSize::NONE;permutation.len()],
         )
         .unwrap();
@@ -423,7 +425,7 @@ impl RearrangeSpec {
         Self::check_rank(count)?;
 
         let out = Self::new(
-            (0..count as u8).map(|i| sv![i]).collect(),
+            (0..count as u8).map(|i| tu8v![i]).collect(),
             permute_combine,
             sv![OpSize::NONE; count],
         )
@@ -446,7 +448,7 @@ impl RearrangeSpec {
                     if axes_to_combine.contains(&z) {
                         None
                     } else {
-                        Some(sv![z])
+                        Some(tu8v![z])
                     }
                 })
                 .chain(std::iter::once(axes_to_combine.clone()))
@@ -619,7 +621,7 @@ impl RearrangeSpec {
 
     pub fn add_batch_dims(&self, batch_rank: usize) -> Self {
         let to_prepend =
-            || (self.int_sizes.len()..self.int_sizes.len() + batch_rank).map(|x| sv![x as u8]);
+            || (self.int_sizes.len()..self.int_sizes.len() + batch_rank).map(|x| tu8v![x as u8]);
         Self::new(
             to_prepend()
                 .chain(self.input_ints.iter().cloned())
@@ -709,24 +711,24 @@ impl RearrangeSpec {
     /// # Examples
     /// 1. if special_case_ones, remove all indices of size 1
     /// ```
-    /// # use rr_util::{sv, rearrange_spec::{OpSize, RearrangeSpec}};
+    /// # use rr_util::{sv,tu8v, rearrange_spec::{OpSize, RearrangeSpec}};
     /// let spec = RearrangeSpec::new(
-    ///     sv![sv![0, 1], sv![2, 3]],
-    ///     sv![sv![3, 2, 1, 0]],
+    ///     sv![tu8v![0, 1], tu8v![2, 3]],
+    ///     sv![tu8v![3, 2, 1, 0]],
     ///     sv![OpSize(3), OpSize(1), OpSize(1), OpSize(1)],
     /// )
     /// .unwrap();
     /// let canonicalized_spec =
-    ///     RearrangeSpec::new(sv![sv![0], sv![]], sv![sv![0]], sv![OpSize(3)]).unwrap();
+    ///     RearrangeSpec::new(sv![tu8v![0], tu8v![]], sv![tu8v![0]], sv![OpSize(3)]).unwrap();
     /// assert_eq!(spec.canonicalize(true), canonicalized_spec);
     /// ```
     /// 2. Merge all sequences of indices that always appear together in the same order inside parentheses
     /// 3. Renumber the int indices sequentially based on when they first appear, inputs first then outputs
     /// ```
-    /// # use rr_util::{sv, rearrange_spec::{OpSize, RearrangeSpec}};
+    /// # use rr_util::{sv,tu8v, rearrange_spec::{OpSize, RearrangeSpec}};
     /// let spec = RearrangeSpec::new(
-    ///     sv![sv![0], sv![1, 2], sv![3, 4, 5]],
-    ///     sv![sv![1, 2], sv![5], sv![3, 4], sv![0]],
+    ///     sv![tu8v![0], tu8v![1, 2], tu8v![3, 4, 5]],
+    ///     sv![tu8v![1, 2], tu8v![5], tu8v![3, 4], tu8v![0]],
     ///     sv![
     ///         OpSize(2),
     ///         OpSize(2),
@@ -738,8 +740,8 @@ impl RearrangeSpec {
     /// )
     /// .unwrap();
     /// let canonicalized_spec = RearrangeSpec::new(
-    ///     sv![sv![0], sv![1], sv![2, 3]],
-    ///     sv![sv![1], sv![3], sv![2], sv![0]],
+    ///     sv![tu8v![0], tu8v![1], tu8v![2, 3]],
+    ///     sv![tu8v![1], tu8v![3], tu8v![2], tu8v![0]],
     ///     sv![OpSize(2), OpSize(4), OpSize(4), OpSize(2)],
     /// )
     /// .unwrap();
@@ -828,7 +830,7 @@ impl RearrangeSpec {
         let mut merge_tuples_and_renumber_fn = |ints: RInts| {
             ints.iter()
                 .map(|single_dim| {
-                    let mut new_dim = sv![];
+                    let mut new_dim: TinyVecU8 = tu8v![];
                     for &i in single_dim.iter() {
                         if let Some(&new_i) = map.get(&i) {
                             new_dim.push(new_i);
@@ -877,7 +879,7 @@ impl RearrangeSpec {
             .map(|ints| {
                 if ints.is_empty() {
                     next_int += 1;
-                    sv![next_int - 1]
+                    tu8v![next_int - 1]
                 } else {
                     ints.clone()
                 }
@@ -890,7 +892,7 @@ impl RearrangeSpec {
             .map(|ints| {
                 if ints.is_empty() {
                     next_int += 1;
-                    sv![next_int - 1]
+                    tu8v![next_int - 1]
                 } else {
                     ints.clone()
                 }
@@ -898,7 +900,9 @@ impl RearrangeSpec {
             .collect();
         if int_after_input > first_int_to_add {
             if !output_ints.is_empty() {
-                output_ints[0].extend(first_int_to_add..int_after_input);
+                for i in first_int_to_add..int_after_input {
+                    output_ints[0].push(i);
+                }
             } else if !allow_rust_invalid {
                 bail!(RearrangeSpecError::NotConvertable { spec: self.clone() });
             }
@@ -1228,9 +1232,9 @@ impl RearrangeSpec {
         RearrangeSpec::new(
             (0..output_shape.len())
                 .filter(|i| !removed_axes.contains(i))
-                .map(|i| sv![i as u8])
+                .map(|i| tu8v![i as u8])
                 .collect(),
-            (0..output_shape.len()).map(|i| sv![i as u8]).collect(),
+            (0..output_shape.len()).map(|i| tu8v![i as u8]).collect(),
             shape_to_op_shape(output_shape),
         )
         .unwrap()
@@ -1244,8 +1248,8 @@ impl RearrangeSpec {
         let int_sizes_usize: HashMap<usize, usize> =
             int_sizes.iter().map(|(k, v)| (*k as usize, *v)).collect();
         RearrangeSpec::new(
-            input_ints.iter().map(|i| sv![*i]).collect(),
-            output_ints.iter().map(|i| sv![*i]).collect(),
+            input_ints.iter().map(|i| tu8v![*i]).collect(),
+            output_ints.iter().map(|i| tu8v![*i]).collect(),
             shape_to_op_shape(&dict_to_list(&int_sizes_usize, None).into_iter().collect()),
         )
     }
@@ -1277,18 +1281,22 @@ impl RearrangeSpec {
     ///
     /// # Examples
     /// ```
-    /// # use rr_util::{sv, rearrange_spec::{OpSize, RearrangeSpec}};
+    /// # use rr_util::{sv,tu8v, rearrange_spec::{OpSize, RearrangeSpec}};
     /// let inner = RearrangeSpec::new(
-    ///     sv![sv![0], sv![1]],
-    ///     sv![sv![0, 1]],
+    ///     sv![tu8v![0], tu8v![1]],
+    ///     sv![tu8v![0, 1]],
     ///     sv![OpSize(2), OpSize(3)],
     /// )
     /// .unwrap();
-    /// let outer =
-    ///     RearrangeSpec::new(sv![sv![0]], sv![sv![0], sv![1]], sv![OpSize(6), OpSize(4)]).unwrap();
+    /// let outer = RearrangeSpec::new(
+    ///     sv![tu8v![0]],
+    ///     sv![tu8v![0], tu8v![1]],
+    ///     sv![OpSize(6), OpSize(4)],
+    /// )
+    /// .unwrap();
     /// let fused = RearrangeSpec::new(
-    ///     sv![sv![0], sv![1]],
-    ///     sv![sv![0, 1], sv![2]],
+    ///     sv![tu8v![0], tu8v![1]],
+    ///     sv![tu8v![0, 1], tu8v![2]],
     ///     sv![OpSize(2), OpSize(3), OpSize(4)],
     /// )
     /// .unwrap();
@@ -1497,8 +1505,8 @@ impl RearrangeSpec {
         assert_eq!(next_sorted_axis, sorted_axes.len());
 
         let out = Self::new(
-            input_ints.into_iter().map(|i| sv![i as u8]).collect(),
-            output_ints.into_iter().map(|i| sv![i as u8]).collect(),
+            input_ints.into_iter().map(|i| tu8v![i as u8]).collect(),
+            output_ints.into_iter().map(|i| tu8v![i as u8]).collect(),
             std::iter::repeat(OpSize::NONE)
                 .take(orig_ndim)
                 .map(Ok)
