@@ -54,15 +54,18 @@ impl Default for SymbolicSizeProduct {
 impl From<usize> for SymbolicSizeProduct {
     // TODO: make this faster as needed
     fn from(value: usize) -> Self {
-        if value == 0 {
+        if value < SYMBOLIC_SIZES[0] {
             return SymbolicSizeProduct {
-                other_factor: 0,
+                other_factor: value,
                 symbolic_sizes: Vec::new(),
             };
         }
         let mut running_value = value;
         let mut symbolic_sizes = Vec::new();
         for (i, p) in SYMBOLIC_SIZES.iter().enumerate() {
+            if running_value < *p {
+                break;
+            }
             if running_value % p == 0 {
                 symbolic_sizes.push(i);
                 running_value /= p;
