@@ -54,6 +54,9 @@ pub fn broadcast_shapes_impl(
     if shapes.is_empty() {
         return Ok(sv![]);
     }
+    if shapes.len() == 1 {
+        return Ok(shapes[0].borrow().iter().cloned().collect());
+    }
     let ranks: Vec<usize> = shapes.iter().map(|x| x.borrow().len()).collect();
     let out_rank = *ranks.iter().max().unwrap();
     let mut existing = vec![None; out_rank];
@@ -1037,6 +1040,7 @@ string_enum! {
     TorchDevice,
     (
         Cpu="cpu",
+        Cuda="cuda",
         Cuda0="cuda:0",
         Cuda1="cuda:1",
         Cuda2="cuda:2",
