@@ -13,14 +13,13 @@ use macro_rules_attribute::apply;
 use once_cell::sync::Lazy;
 use pyo3::{types::PyBytes, FromPyObject, IntoPy, PyObject, Python};
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
-use smallvec::SmallVec;
 
-use crate::{make_lazy, simple_from};
+use crate::{compact_data::TinyVecU8, make_lazy, simple_from};
 pub type AxisInt = u8;
 pub type HashBytes = [u8; 32];
 // we could use u64 instead of u8 if we wanted.
 // this is u8, so we want to use multiple of 8 inline so we don't pad
-pub type EinsumAxes = SmallVec<[AxisInt; 8]>;
+pub type EinsumAxes = TinyVecU8;
 pub type NamedAxes = BTreeMap<AxisInt, String>;
 
 #[derive(FromPyObject, Copy, Clone, Debug, Eq, PartialEq, Ord, PartialOrd)]
@@ -669,6 +668,12 @@ pub fn indent(x: String, width: usize) -> String {
     x.lines().map(|s| tab.clone() + s).join("\n")
 }
 
+pub fn indent_butfirst(x: String, width: usize) -> String {
+    let tab = " ".repeat(width) + "\n";
+
+    x.lines().join(&tab)
+}
+
 /// https://stackoverflow.com/questions/64498617/how-to-transpose-a-vector-of-vectors-in-rust
 pub fn transpose<T>(v: Vec<Vec<T>>, inner_len: usize) -> Vec<Vec<T>> {
     for x in &v {
