@@ -3,8 +3,8 @@ from pdb import post_mortem
 import torch
 from rust_circuit import (
     Einsum,
-    ScalarConstant,
-    ArrayConstant,
+    Scalar,
+    Array,
     Symbol,
     Add,
     Rearrange,
@@ -23,8 +23,8 @@ from rust_circuit import (
 # %%
 
 base_add = Add(
-    ScalarConstant(0.2, name="hi"),
-    ScalarConstant(0.2, name="hi"),
+    Scalar(0.2, name="hi"),
+    Scalar(0.2, name="hi"),
 )
 
 nested_add = Add(base_add, base_add)
@@ -46,8 +46,8 @@ elimed.print()
 
 scalar_merged = add_collapse_scalar_inputs(
     Add(
-        ScalarConstant(171),
-        ScalarConstant(9),
+        Scalar(171),
+        Scalar(9),
     )
 )
 assert scalar_merged is not None
@@ -55,23 +55,23 @@ scalar_merged.print()
 
 scalar_merged_call = add_collapse_scalar_inputs(
     Add(
-        ScalarConstant(171),
-        ScalarConstant(9),
+        Scalar(171),
+        Scalar(9),
     )
 )
 assert scalar_merged_call is not None
 scalar_merged_call.print()
 
 add_with_rearrange = Add(
-    Rearrange(ScalarConstant(2), RearrangeSpec([], [[0], [1]], [2, 3])),
-    Rearrange(ScalarConstant(2, (2,)), RearrangeSpec([[0]], [[0], [1]], [2, 3])),
+    Rearrange(Scalar(2), RearrangeSpec([], [[0], [1]], [2, 3])),
+    Rearrange(Scalar(2, (2,)), RearrangeSpec([[0]], [[0], [1]], [2, 3])),
 )
 post_rearrange = add_pull_removable_axes(add_with_rearrange, True)
 assert post_rearrange is not None
 post_rearrange.print()
 # %%
 
-ein = Einsum((ScalarConstant(2, (2, 3)), (0, 1)), (ScalarConstant(3, (3, 4)), (1, 2)), out_axes=(0, 2))
+ein = Einsum((Scalar(2, (2, 3)), (0, 1)), (Scalar(3, (3, 4)), (1, 2)), out_axes=(0, 2))
 ein.print()
 ein_deep = Einsum((ein, (0, 1)), (ein, (0, 1)), out_axes=(0,))
 ein_deep.print()
@@ -88,16 +88,16 @@ assert distributed is not None
 distributed.print()
 
 for_pull = Einsum(
-    (Rearrange(ScalarConstant(2), RearrangeSpec([], [[0], [1]], [2, 3])), (0, 1)),
-    (Rearrange(ScalarConstant(2, (2,)), RearrangeSpec([[0]], [[0], [1]], [2, 3])), (0, 1)),
+    (Rearrange(Scalar(2), RearrangeSpec([], [[0], [1]], [2, 3])), (0, 1)),
+    (Rearrange(Scalar(2, (2,)), RearrangeSpec([[0]], [[0], [1]], [2, 3])), (0, 1)),
     out_axes=(0, 1),
 )
 pulled = einsum_pull_removable_axes(for_pull)
 assert pulled is not None
 pulled.print()
 
-rearrange_identity = Rearrange(ScalarConstant(2, (2,)), RearrangeSpec([[0]], [[0]], [2]))
-rearrange_identity_2 = Rearrange(ScalarConstant(2, (1, 1)), RearrangeSpec([[0], [1]], [[1], [0]], [1, 1]))
+rearrange_identity = Rearrange(Scalar(2, (2,)), RearrangeSpec([[0]], [[0]], [2]))
+rearrange_identity_2 = Rearrange(Scalar(2, (1, 1)), RearrangeSpec([[0], [1]], [[1], [0]], [1, 1]))
 print(rearrange_identity.spec.is_identity())
 print(rearrange_identity.spec.is_identity())
 print(rearrange_identity_2.spec.is_identity())
@@ -105,7 +105,7 @@ print(rearrange_identity_2.spec.is_identity())
 
 # %%
 
-r1 = Rearrange(ArrayConstant(torch.randn(2, 15)), RearrangeSpec([[0], [1]], [[0, 1]], [2, 15]))
+r1 = Rearrange(Array(torch.randn(2, 15)), RearrangeSpec([[0], [1]], [[0, 1]], [2, 15]))
 r2 = Rearrange(r1, RearrangeSpec([[0, 1]], [[0], [1]], [10, 3]))
 r1.node
 
